@@ -16,6 +16,12 @@
 #include <gck/gck.h>
 #include <gcr/gcr.h>
 
+#if !GCR_CHECK_VERSION(3,90,0)
+#define gck_slot_open_session_async(self, options, interaction, cancellable, callback, user_data) \
+	gck_slot_open_session_async(self, options, cancellable, callback, user_data)
+#define gck_uri_data_build gck_uri_build
+#endif
+
 /**
  * SECTION:nma-pkcs11-cert-chooser-dialog
  * @title: NMAPkcs11CertChooserDialog
@@ -50,6 +56,8 @@ struct _NMAPkcs11CertChooserDialogPrivate {
 	GtkCellRenderer *list_name_renderer;
 	GtkTreeViewColumn *list_issued_by_column;
 	GtkCellRenderer *list_issued_by_renderer;
+
+	gboolean object_selected;
 };
 
 #define NMA_PKCS11_CERT_CHOOSER_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
@@ -117,7 +125,8 @@ object_details (GObject *source_object, GAsyncResult *res, gpointer user_data)
 	CK_OBJECT_CLASS cka_class;
 	const GckAttribute *attr;
 	GcrCertificate *cert;
-	gchar *label, *issuer;
+	gchar *label = NULL;
+	gchar *issuer = NULL;
 	GError *error = NULL;
 	GtkListStore *store1, *store2;
 	IdMatchData data;
@@ -155,24 +164,27 @@ object_details (GObject *source_object, GAsyncResult *res, gpointer user_data)
 	                        id_match,
 	                        &data);
 
+	attr = gck_attributes_find (attrs, CKA_LABEL);
+	if (attr && attr->value && attr->length) {
+		label = g_malloc (attr->length + 1);
+		memcpy (label, attr->value, attr->length);
+		label[attr->length] = '\0';
+	}
+
 	attr = gck_attributes_find (attrs, CKA_VALUE);
 	if (attr && attr->value && attr->length) {
 		cert = gcr_simple_certificate_new (attr->value, attr->length);
-		label = gcr_certificate_get_subject_name (cert);
+		if (!label)
+			label = gcr_certificate_get_subject_name (cert);
 		issuer = gcr_certificate_get_issuer_name (cert);
 		g_object_unref (cert);
-	} else {
-		attr = gck_attributes_find (attrs, CKA_LABEL);
-		if (attr && attr->value && attr->length) {
-			label = g_malloc (attr->length + 1);
-			memcpy (label, attr->value, attr->length);
-			label[attr->length] = '\0';
-		} else {
-			label = g_strdup (_("(Unknown)"));
-		}
-		issuer = g_memdup ("", 1);
 	}
 
+	if (!label)
+		label = g_strdup (_("(Unknown)"));
+	if (!issuer)
+		issuer = g_memdup ("", 1);
+
 	gtk_list_store_append (store1, &iter);
 	gtk_list_store_set (store1, &iter,
 	                    COLUMN_LABEL, label,
@@ -283,18 +295,23 @@ static void
 row_activated (GtkTreeView *tree_view, GtkTreePath *path,
                GtkTreeViewColumn *column, gpointer user_data)
 {
-	if (gtk_window_activate_default (GTK_WINDOW (user_data)))
-		return;
+	NMAPkcs11CertChooserDialog *self = user_data;
+	NMAPkcs11CertChooserDialogPrivate *priv = self->priv;
+
+	if (priv->object_selected)
+		gtk_dialog_response(GTK_DIALOG (user_data), GTK_RESPONSE_ACCEPT);
 }
 
 static void
 cursor_changed (GtkTreeView *tree_view, gpointer user_data)
 {
 	NMAPkcs11CertChooserDialog *self = NMA_PKCS11_CERT_CHOOSER_DIALOG (user_data);
+	NMAPkcs11CertChooserDialogPrivate *priv = self->priv;
 	gchar *uri;
 
 	uri = nma_pkcs11_cert_chooser_dialog_get_uri (self);
-	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_ACCEPT, uri != NULL);
+	priv->object_selected = uri != NULL;
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (self), GTK_RESPONSE_ACCEPT, priv->object_selected);
 	g_free (uri);
 }
 
@@ -332,7 +349,7 @@ login_clicked (GtkButton *button, gpointer user_data)
 		priv->pin_length = 0;
 		priv->pin_value =  g_memdup ("", 1);
 		priv->remember_pin = TRUE;
-		gck_slot_open_session_async (priv->slot, GCK_SESSION_READ_ONLY, NULL, session_opened, self);
+		gck_slot_open_session_async (priv->slot, GCK_SESSION_READ_ONLY, NULL, NULL, session_opened, self);
 		return;
 	}
 
@@ -346,7 +363,7 @@ login_clicked (GtkButton *button, gpointer user_data)
 		priv->pin_value = g_memdup (nma_pkcs11_token_login_dialog_get_pin_value (NMA_PKCS11_TOKEN_LOGIN_DIALOG (dialog)),
 		                            priv->pin_length + 1);
 		priv->remember_pin = nma_pkcs11_token_login_dialog_get_remember_pin (NMA_PKCS11_TOKEN_LOGIN_DIALOG (dialog));
-		gck_slot_open_session_async (priv->slot, GCK_SESSION_READ_ONLY, NULL, session_opened, self);
+		gck_slot_open_session_async (priv->slot, GCK_SESSION_READ_ONLY, NULL, NULL, session_opened, self);
 	}
 
 	gtk_window_destroy (GTK_WINDOW (dialog));
@@ -383,7 +400,7 @@ set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *p
 		if ((token_info->flags & CKF_LOGIN_REQUIRED) == 0)
 			gtk_widget_set_sensitive (priv->login_button, FALSE);
 		gck_token_info_free (token_info);
-		gck_slot_open_session_async (priv->slot, GCK_SESSION_READ_ONLY, NULL, session_opened, self);
+		gck_slot_open_session_async (priv->slot, GCK_SESSION_READ_ONLY, NULL, NULL, session_opened, self);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -562,7 +579,7 @@ nma_pkcs11_cert_chooser_dialog_get_uri (NMAPkcs11CertChooserDialog *dialog)
 
 	uri_data.attributes = gck_builder_end (builder);
 	uri_data.token_info = gck_slot_get_token_info (priv->slot);
-	uri = gck_uri_build (&uri_data, GCK_URI_FOR_OBJECT_ON_TOKEN);
+	uri = gck_uri_data_build (&uri_data, GCK_URI_FOR_OBJECT_ON_TOKEN);
 
 	gck_attributes_unref (uri_data.attributes);
 	gck_attributes_unref (attrs);
